@@ -25,6 +25,7 @@ from inferelator.distributed.inferelator_mp import MPControl
 from inferelator.utils import Validator as check
 from inferelator import utils
 from inferelator import workflow
+from inferelator.postprocessing.model_metrics import MetricHandler
 
 
 class CrossValidationManager(object):
@@ -35,7 +36,7 @@ class CrossValidationManager(object):
     # Output settings
     _csv_writer = None  # csv.csvwriter
     _csv_header = None  # list[]
-    output_file_name = "aupr.tsv"  # str
+    output_file_name = "crossvalidation_performance.tsv"  # str
     _csv_writer_object = csv.writer
     _csv_file_handle = None
 
@@ -132,7 +133,16 @@ class CrossValidationManager(object):
             self.grid_params = []
 
         self.grid_params.append(param_name)
-        self.grid_param_values[param_name] = param_vector
+
+        if utils.is_string(param_vector):
+            self.grid_param_values[param_name] = [param_vector]
+        else:
+            try:
+                # There's probably a better way to check and see if something is iterable but I don't care
+                [True for _ in param_vector]
+                self.grid_param_values[param_name] = param_vector
+            except TypeError:
+                self.grid_param_values[param_name] = [param_vector]
 
     def add_grouping_dropout(self, metadata_column_name, group_size=None, seed=42):
         """
@@ -212,22 +222,24 @@ class CrossValidationManager(object):
         self._check_grid_search_params_exist()
 
         # Run base grid search
-        self._grid_search()
+        results = self._grid_search()
 
         # Run size sampling
         if self.size_sample_vector is not None:
-            self._size_cv()
+            results.extend(self._size_cv())
 
         # Run dropin
         if self.dropin_column is not None:
-            self._dropin_cv()
+            results.extend(self._dropin_cv())
 
         # Run dropout
         if self.dropout_column is not None:
-            self._dropout_cv()
+            results.extend(self._dropout_cv())
 
         self._destroy_writer()
 
+        return results
+
     def append_to_path(self, var_name, to_append):
         """
         Add a string to an existing path variable in class
@@ -254,8 +266,8 @@ class CrossValidationManager(object):
             # Add Test & Value columns for dropouts/etc
             self._csv_header.extend(["Test", "Value", "Num_Obs"])
 
-            # Also add the metric
-            self._csv_header.append(self.workflow.metric)
+            # Also add the metric name
+            self._csv_header.extend(MetricHandler.get_metric(self.workflow.metric).all_names())
 
             # Create a CSV writer
             self._create_output_path()
@@ -356,6 +368,8 @@ class CrossValidationManager(object):
         # This is unpacked in the same order that is used in the header
         ordered_unpack = [self.grid_param_values[param] for param in self.grid_params]
 
+        results = []
+
         for param_values in itertools.product(*ordered_unpack):
             params = zip(self.grid_params, param_values)
             csv_line = []
@@ -391,13 +405,19 @@ class CrossValidationManager(object):
             cv_workflow.append_to_path("output_dir", output_path)
             utils.Debug.vprint("Writing results to {p} ".format(p=cv_workflow.output_dir), level=1)
             result = cv_workflow.run()
-            csv_line.extend([test, value, n_obs, result.score])
+
+            # Output the results to the CSV file
+            csv_line.extend([test, value, n_obs] + [result.all_scores[n] for n in result.all_names])
+
+            results.append(((test, value), result))
 
             if MPControl.is_master:
                 self._csv_writer.writerow(csv_line)
 
             del cv_workflow
 
+        return results
+
     def _check_grid_search_params_exist(self):
         """
         Determine if the parameters for grid search are workflow parameters
@@ -444,6 +464,8 @@ class CrossValidationManager(object):
 
         unique_groups = meta_data[col].unique().tolist()
 
+        results = []
+
         # Downsample if max_size is set for a comparable all-group control
         if len(unique_groups) > 2 and max_size is not None:
             rgen = np.random.RandomState(self.dropout_seed - 1)
@@ -457,7 +479,8 @@ class CrossValidationManager(object):
 
                 return include_mask
 
-            self._grid_search(test="dropout", value="all", mask_function=mask_function)
+            result = self._grid_search(test="dropout", value="all", mask_function=mask_function)
+            results.append((("dropout", "all"), result))
 
         # Iterate through groups and drop one
         for i, group in enumerate(unique_groups):
@@ -478,7 +501,10 @@ class CrossValidationManager(object):
 
                     return include_mask
 
-            self._grid_search(test="dropout", value=group, mask_function=mask_function)
+            result = self._grid_search(test="dropout", value=group, mask_function=mask_function)
+            results.append((("dropout", group), result))
+
+        return results
 
     def _dropin_cv(self):
         """
@@ -491,13 +517,16 @@ class CrossValidationManager(object):
 
         unique_groups = meta_data[col].unique().tolist()
 
+        results = []
+
         if len(unique_groups) > 1 and max_size:
             rgen = np.random.RandomState(self.dropin_seed - 1)
 
             def mask_function():
                 return group_index(meta_data, col, group=None, rgen=rgen, max_size=max_size)
 
-            self._grid_search(test="dropin", value="all", mask_function=mask_function)
+            result = self._grid_search(test="dropin", value="all", mask_function=mask_function)
+            results.append((("dropin", "all"), result))
 
         for i, group in enumerate(unique_groups):
             rgen = np.random.RandomState(self.dropin_seed + i)
@@ -508,13 +537,18 @@ class CrossValidationManager(object):
                 else:
                     return group_index(meta_data, col, group, rgen=rgen, max_size=max_size)
 
-            self._grid_search(test="dropin", value=group, mask_function=mask_function)
+            result = self._grid_search(test="dropin", value=group, mask_function=mask_function)
+            results.append((("dropin", group), result))
+
+        return results
 
     def _size_cv(self):
         """
         Run grid search on a subset of the data
         """
 
+        results = []
+
         for i, size_ratio in enumerate(self.size_sample_vector):
             rgen = np.random.RandomState(self.size_sample_seed + i)
             meta_data = self.workflow.meta_data.copy()
@@ -542,7 +576,10 @@ class CrossValidationManager(object):
                     rgen.shuffle(size_mask)
                     return pd.Series(size_mask, index=meta_data.index)
 
-            self._grid_search(test="size", value=str(size_ratio), mask_function=data_masker)
+            result = self._grid_search(test="size", value=str(size_ratio), mask_function=data_masker)
+            results.append((("size", size_ratio), result))
+
+        return results
 
 
 def group_index(meta, meta_col, group=None, size_ratio=None, rgen=None, max_size=None):
